/*
 *  Copyright (C) 2010 by Filip Brcic <brcha@gna.org>
 *
 *  This file is part of VukoviTray
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "vukovitray.h"
#include "ui_vukovitray.h"

#include <QtCore/QSettings>
#include <QtCore/QTimer>

#include <QtGui/QMenu>
#include <QtGui/QCloseEvent>
#include <QtGui/QMessageBox>
#include <QtGui/QDesktopServices>

#include <QtNetwork/QNetworkAccessManager>
#include <QtNetwork/QNetworkReply>
#include <QtNetwork/QNetworkRequest>

VukoviTray::VukoviTray(QWidget *parent)
    : QWidget(parent), ui(new Ui::VukoviTray)
{
    ui->setupUi(this);

    createTrayIcon();

    setupActions();

    loadSettings();

    order = QString::fromUtf8("Прво подесите систем");

    networkAccessManager = new QNetworkAccessManager(this);
    connect(networkAccessManager, SIGNAL(finished(QNetworkReply*)), this, SLOT(receiveOrders(QNetworkReply*)));

    trayIcon->show();

//    // Check for new version
//    updateCheckUrl = QUrl("http://bitbucket.org/brcha/vukovitray/raw/tip/.latest");
//    networkAccessManager->get(QNetworkRequest(updateCheckUrl));

//    connect(this, SIGNAL(newVersionAvailable(QString)), this, SLOT(detectedNewVersion(QString)));
}

VukoviTray::~VukoviTray()
{
    delete ui;
}

void VukoviTray::loadSettings()
{
    settings = new QSettings("Vukovi", "Vucija naredjenja", this);

    if (!settings->contains("orders_url") || !settings->contains("update_interval"))
    {
        this->show();
        updateInterval = -1;
    }
    else
    {
        ordersUrl = settings->value("orders_url").toUrl();
        updateInterval = settings->value("update_interval").toInt();
        updateIntervalSecs = settings->value("update_interval_seconds").toInt();
        ui->edt_order_url->setText(ordersUrl.toString());
        ui->sb_minutes->setValue(updateInterval);
        ui->sb_seconds->setValue(updateIntervalSecs);
        // Wait 10 seconds before starting update
        QTimer::singleShot(10000, this, SLOT(startOrdersUpdate()));
    }
}

void VukoviTray::saveSettings()
{
    if (updateInterval != -1)
    {
        settings->setValue("orders_url", ordersUrl);
        settings->setValue("update_interval", updateInterval);
        settings->setValue("update_interval_seconds", updateIntervalSecs);
    }
}

void VukoviTray::closeEvent(QCloseEvent * event)
{
    if (trayIcon->isVisible())
    {
        hide();
        event->ignore();
    }
}

void VukoviTray::createTrayIcon()
{
    QMenu * trayMenu = new QMenu(this);
    trayMenu->addAction(ui->action_show_message);
    trayMenu->addAction(ui->action_setup);
    trayMenu->addSeparator();
    trayMenu->addAction(ui->action_exit);

    trayIcon = new QSystemTrayIcon(this);
    trayIcon->setContextMenu(trayMenu);

    trayIcon->setIcon(QIcon(":/images/logo.jpg"));
}

void VukoviTray::setupActions()
{
    connect(ui->btn_save, SIGNAL(clicked()), this, SLOT(hideWindow()));
    connect(ui->action_exit, SIGNAL(triggered()), this, SLOT(quitApplication()));
    connect(ui->action_show_message, SIGNAL(triggered()), this, SLOT(showMessage()));
    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
            this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
}

void VukoviTray::showMessage()
{
    trayIcon->setToolTip(order);
    trayIcon->showMessage(QString::fromUtf8("Наређење"), order,
                          QSystemTrayIcon::Information,
                          60000);
}

void VukoviTray::hideWindow()
{
    ordersUrl = QUrl(ui->edt_order_url->text());
    if (!ordersUrl.isValid())
    {
        QMessageBox::critical(this, QString::fromUtf8("Погрешна адреса"),
                              QString::fromUtf8("Адреса за наређења није валидна."));
        return;
    }
    updateInterval = ui->sb_minutes->value();
    updateIntervalSecs = ui->sb_seconds->value();
    saveSettings();
    hide();

    // Update orders
    startOrdersUpdate();
}

void VukoviTray::quitApplication()
{
    saveSettings();
    qApp->quit();
}

void VukoviTray::startOrdersUpdate()
{
    networkAccessManager->get(QNetworkRequest(ordersUrl));
}

//void VukoviTray::detectedNewVersion(QString version)
//{
//    show();
//    QMessageBox::information(this, QString::fromUtf8("Нова верзија"),
//                             QString::fromUtf8("Доступна је нова верзија %1.\n"
//                                               "Скините нову верзију са сајта http://bitbucket.org/brcha/vukovitray/downloads").arg(version));
//    QDesktopServices::openUrl(QUrl("http://bitbucket.org/brcha/vukovitray/downloads"));
//    hide();
//}

void VukoviTray::receiveOrders(QNetworkReply * reply)
{
    QString newOrder = QString::fromUtf8(reply->readAll());

//    if (reply->url() == updateCheckUrl)
//    {
//        if (newOrder.trimmed() != VERSION)
//            emit newVersionAvailable(newOrder.trimmed());
//    }
//    else
//    {
        bool shouldShowMessage = false;
        if (newOrder != order)
            shouldShowMessage = true;

        order = newOrder;

        if (shouldShowMessage)
            showMessage();

        // Start next round
        QTimer::singleShot(updateInterval*60000 + updateIntervalSecs*1000, this, SLOT(startOrdersUpdate()));
//    }
}

void VukoviTray::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason)
    {
        case QSystemTrayIcon::Trigger:
        case QSystemTrayIcon::DoubleClick:
            show();
            break;
        case QSystemTrayIcon::MiddleClick:
            showMessage();
            break;
        default:
            break;
    }
}
